from django.forms import widgets


class GndAcWidget(widgets.Select):

    class Media:
        css = {
            'all': ('https://cdn.jsdelivr.net/npm/select2@4.1.0-rc.0/dist/css/select2.min.css',)
        }
        js = (
            'https://code.jquery.com/jquery-3.5.1.min.js',
            'https://cdn.jsdelivr.net/npm/select2@4.1.0-rc.0/dist/js/select2.min.js',
            'gnd/lobid-ac.js',
        )

    def update_attrs(self, options, attrs):
        attrs = self.fix_class(attrs)
        for key, val in options.items():
            attrs['data-{}'.format(key)] = val

        return attrs

    def fix_class(self, attrs):
        class_name = attrs.pop('class', '')
        if class_name:
            class_name = f"{class_name} {'custom-select'}"
        else:
            class_name = 'custom-select'
        attrs['class'] = class_name

        return attrs

    def __init__(self, attrs=None, *args, **kwargs):

        attrs = attrs or {}
        options = kwargs.pop('options', {})
        new_attrs = self.update_attrs(options, attrs)
        super().__init__(new_attrs)



from datetime import datetime
from dateutil.parser import parse, ParserError
from django.db import models
from pylobid.pylobid import PyLobidClient, PyLobidPerson
from . fields import GndField
from . utils import fetch_gender


class GndBaseModel(models.Model):
    gnd_gnd_id = GndField(
        blank=True, null=True, unique=True
    )
    gnd_pref_name = models.CharField(
        blank=True, null=True, max_length=250
    )
    gnd_payload = models.JSONField(blank=True, null=True)
    gnd_created = models.DateTimeField(
        blank=True, null=True, auto_now=False, auto_now_add=False
    )

    def __str__(self):
        return f"{self.gnd_pref_name} <{self.gnd_gnd_id}>"

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self.gnd_gnd_id and not self.gnd_created:
            py_ent = PyLobidClient(self.gnd_gnd_id, fetch_related=True).factory()
            self.gnd_pref_name = py_ent.pref_name
            self.gnd_payload = py_ent.ent_dict
            self.gnd_created = datetime.now()
        super().save(*args, **kwargs)

    def gnd_thumbnail_url(self):
        try:
            return self.gnd_payload['depiction'][0]['thumbnail']
        except (TypeError, KeyError):
            return None

    def gnd_html_thumb(self):
        if self.gnd_thumbnail_url():
            th_url = self.gnd_thumbnail_url()
            thumb = f'<img src="{th_url}" alt="{self.gnd_pref_name}" class="img-thumbnail">'
            return thumb
        else:
            return None


class GndPersonBase(GndBaseModel):
    gnd_gender = models.CharField(
        blank=True, null=True, max_length=250
    )
    gnd_birth_date_written = models.CharField(
        blank=True, null=True, max_length=250
    )
    gnd_death_date_written = models.CharField(
        blank=True, null=True, max_length=250
    )
    gnd_birth_date = models.DateField(
        blank=True, null=True
    )
    gnd_death_date = models.DateField(
        blank=True, null=True
    )

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if self.gnd_gnd_id and not self.gnd_created:
            py_ent = PyLobidClient(self.gnd_gnd_id, fetch_related=False).factory()
            if isinstance(py_ent, PyLobidPerson):
                self.gnd_birth_date_written = py_ent.life_span['birth_date_str']
                self.gnd_death_date_written = py_ent.life_span['death_date_str']
                try:
                    self.gnd_birth_date = parse(self.gnd_birth_date_written)
                except ParserError:
                    pass
                try:
                    self.gnd_death_date = parse(self.gnd_death_date_written)
                except ParserError:
                    pass
                self.gnd_gender = fetch_gender(self.gnd_payload)
        super().save(*args, **kwargs)



from django.db import models

from . forms import GndFormField


class GndField(models.CharField):

    description = "A Field for a GND Identifier"

    def formfield(self, **kwargs):
        defaults = {
            'form_class': GndFormField,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 250
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs["max_length"]
        return name, path, args, kwargs



from django import forms
from gnd.widgets import GndAcWidget


class GndFormField(forms.CharField):
    widget = GndAcWidget(
        options={
            'placeholder': 'Search the GND',
            'multiple': False,
            'maximum-selection-length': 21,
            'minimumInputLength': 3,
        }
    )


class GndForm(forms.Form):
    gnd_gnd_id = GndFormField(
        label='GND',
        widget=GndAcWidget(
            options={'placeholder': 'Search the GND'}
        ),
    )


class GndModelForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        super(GndModelForm, self).__init__(*args, **kwargs)
        if kwargs['instance']:
            try:
                initial = kwargs['instance'].gnd_gnd_id.split('/')[-1]
            except AttributeError:
                initial = None
            self.fields['hidden_gnd'] = forms.CharField(
                initial=initial,
                required=False,
                widget=forms.HiddenInput()
            )
        else:
            self.fields['hidden_gnd'] = forms.CharField(
                required=False,
                widget=forms.HiddenInput()
            )



GENDER_DEFAULT = [
    {'id': 'https://d-nb.info/standards/vocab/gnd/gender#notKnown'}
]


def fetch_gender(gnd_payload):
    try:
        gender_list = gnd_payload.get(
            'gender', GENDER_DEFAULT
        )
    except AttributeError:
        return GENDER_DEFAULT[0]['id']
    return gender_list[0]['id']



import setuptools
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setuptools.setup(
    name='django-gnd',
    version='0.3.0',
    description="""A django package to query and store data from Lobid's GND-API""",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Peter Andorfer',
    author_email='peter.andorfer@oeaw.ac.at',
    url='https://github.com/acdh-oeaw/django-gnd',
    packages=[
        'gnd',
    ],
    include_package_data=True,
    install_requires=[
        'Django>=3.1',
        'pylobid>=1.3.1',
        'python-dateutil>=2.8'
    ],
    license="MIT",
    zip_safe=False,
    keywords='django-gnd'
)



from django.views.generic import TemplateView

from gnd.forms import GndForm


class GndFormView(TemplateView):

    template_name = 'gnd/tryout.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['form'] = GndForm
        return context
